@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """
@@ -43,16 +43,24 @@ from nuitka.ModuleRegistry import (
     getModuleInclusionInfoByName,
 )
 from nuitka.Options import (
-    hasPythonFlagNoAnnotations,
-    hasPythonFlagNoAsserts,
-    hasPythonFlagNoDocStrings,
+    getCompanyName,
+    getFileVersion,
+    getProductFileVersion,
+    getProductName,
+    getProductVersion,
     isDeploymentMode,
+    isOnefileMode,
+    isOnefileTempDirMode,
     isStandaloneMode,
     shallCreateAppBundle,
     shallMakeModule,
     shallShowExecutedCommands,
 )
-from nuitka.PythonFlavors import isAnacondaPython, isDebianPackagePython
+from nuitka.PythonFlavors import (
+    isAnacondaPython,
+    isDebianPackagePython,
+    isNuitkaPython,
+)
 from nuitka.PythonVersions import (
     getTestExecutionPythonVersions,
     python_version,
@@ -66,7 +74,11 @@ from nuitka.utils.Distributions import (
     getDistributionName,
     isDistributionCondaPackage,
 )
-from nuitka.utils.Execution import NuitkaCalledProcessError, check_output
+from nuitka.utils.Execution import (
+    NuitkaCalledProcessError,
+    check_output,
+    withEnvironmentVarsOverridden,
+)
 from nuitka.utils.FileOperations import (
     changeFilenameExtension,
     getFileContents,
@@ -80,6 +92,7 @@ from nuitka.utils.ModuleNames import (
 )
 from nuitka.utils.SharedLibraries import locateDLL, locateDLLsInDirectory
 from nuitka.utils.SlotMetaClasses import getMetaClassBase
+from nuitka.utils.StaticLibraries import getSystemStaticLibPythonPath
 from nuitka.utils.Utils import (
     getArchitecture,
     isAndroidBasedLinux,
@@ -88,7 +101,6 @@ from nuitka.utils.Utils import (
     isWin32Windows,
     withNoWarning,
 )
-from nuitka.utils.Yaml import getYamlPackageConfiguration
 
 _warned_unused_plugins = set()
 
@@ -115,6 +127,15 @@ def _getImportLibModule():
         return importlib
 
 
+def _makeEvaluationContext(logger, full_name, config_name):
+    context = TagContext(logger=logger, full_name=full_name, config_name=config_name)
+    context.update(control_tags)
+
+    context.update(_getEvaluationContext())
+
+    return context
+
+
 def _getEvaluationContext():
     # Using global here, as this is really a singleton, in the form of a module,
     # pylint: disable=global-statement
@@ -129,22 +150,33 @@ def _getEvaluationContext():
             "android32": isAndroidBasedLinux() and sys.maxsize < 2**32,
             "android64": isAndroidBasedLinux() and sys.maxsize >= 2**64 - 1,
             "anaconda": isAnacondaPython(),
-            "is_conda_package": isDistributionCondaPackage,
+            "is_conda_package": _isCondaPackage,
             "debian_python": isDebianPackagePython(),
+            "nuitka_python": isNuitkaPython(),
             "standalone": isStandaloneMode(),
+            "onefile": isOnefileMode(),
+            "onefile_cached": not isOnefileTempDirMode(),
             "module_mode": shallMakeModule(),
             "deployment": isDeploymentMode(),
+            # Version information
+            "company": getCompanyName(),
+            "product": getProductName(),
+            "file_version": getFileVersion(),
+            "product_version": getProductVersion(),
+            "combined_version": getProductFileVersion(),
             # Querying package versions.
             "version": _getPackageVersion,
+            "version_str": _getPackageVersionStr,
             "get_dist_name": _getDistributionNameFromPackageName,
             "plugin": _isPluginActive,
-            "no_asserts": hasPythonFlagNoAsserts(),
-            "no_docstrings": hasPythonFlagNoDocStrings(),
-            "no_annotations": hasPythonFlagNoAnnotations(),
             # Iterating packages
             "iterate_modules": _iterate_module_names,
             # Locating package directories
-            "_get_module_directory": _getModuleDirectory,
+            "get_module_directory": _getModuleDirectory,
+            # Checking module presence
+            "has_module": _hasModule,
+            # Getting data files contents
+            "get_data": _getPackageData,
             # Querying package properties
             "has_builtin_module": isBuiltinModuleName,
             # Architectures
@@ -160,6 +192,8 @@ def _getEvaluationContext():
             # Python version string
             "python_version_str": python_version_str,
             "python_version_full_str": python_version_full_str,
+            # Technical requirements
+            "static_libpython": getSystemStaticLibPythonPath() is not None,
             # Builtins
             "True": True,
             "False": False,
@@ -186,16 +220,19 @@ def _getEvaluationContext():
             numeric_version = int(big) * 256 + int(major) * 16
             is_same_or_higher_version = python_version >= numeric_version
 
-            _context_dict[
-                "python" + big + major + "_or_higher"
-            ] = is_same_or_higher_version
+            _context_dict["python" + big + major + "_or_higher"] = (
+                is_same_or_higher_version
+            )
             _context_dict["before_python" + big + major] = not is_same_or_higher_version
 
         _context_dict["before_python3"] = python_version < 0x300
         _context_dict["python3_or_higher"] = python_version >= 0x300
 
-        _context_dict["extension_std_suffix"] = getSharedLibrarySuffix(preferred=True)
-        _context_dict["extension_suffix"] = getSharedLibrarySuffix(preferred=False)
+        if not isNuitkaPython():
+            _context_dict["extension_std_suffix"] = getSharedLibrarySuffix(
+                preferred=True
+            )
+            _context_dict["extension_suffix"] = getSharedLibrarySuffix(preferred=False)
 
     return _context_dict
 
@@ -209,6 +246,8 @@ def _convertVersionToTuple(version_str):
 
 
 def _getPackageNameFromDistributionName(distribution_name):
+    # spell-checker: ignore opencv, pyobjc, objc
+
     if distribution_name in ("opencv-python", "opencv-python-headless"):
         return "cv2"
     elif distribution_name == "pyobjc":
@@ -227,6 +266,12 @@ def _getDistributionNameFromPackageName(package_name):
         return getDistributionName(distribution)
 
 
+def _isCondaPackage(package_name):
+    distribution_name = _getDistributionNameFromPackageName(package_name)
+
+    return isDistributionCondaPackage(distribution_name)
+
+
 def _getPackageVersion(distribution_name):
     if distribution_name not in _package_versions:
         try:
@@ -272,6 +317,15 @@ def _getPackageVersion(distribution_name):
     return _package_versions[distribution_name]
 
 
+def _getPackageVersionStr(distribution_name):
+    version = _getPackageVersion(distribution_name)
+
+    if version is not None:
+        version = ".".join(str(d) for d in version)
+
+    return version
+
+
 def _getModuleDirectory(module_name):
     from nuitka.importing.Importing import locateModule
 
@@ -282,6 +336,22 @@ def _getModuleDirectory(module_name):
     return module_filename
 
 
+def _hasModule(module_name):
+    from nuitka.importing.Importing import locateModule
+
+    _module_name, _module_filename, _module_kind, finding = locateModule(
+        module_name=ModuleName(module_name), parent_package=None, level=0
+    )
+
+    return finding != "not-found"
+
+
+def _getPackageData(package_name, resource):
+    from nuitka.utils.PackageResources import getPackageData
+
+    return getPackageData(package_name=ModuleName(package_name), resource=resource)
+
+
 def _iterate_module_names(package_name):
     package_name = ModuleName(package_name)
     package_path = _getModuleDirectory(module_name=package_name)
@@ -292,7 +362,7 @@ def _iterate_module_names(package_name):
         module_name = package_name.getChildNamed(module_info.name)
         result.append(module_name.asString())
 
-        if module_info.ispkg:
+        if module_info.ispkg:  # spell-checker: ignore ispkg
             result.extend(_iterate_module_names(package_name=module_name))
 
     return result
@@ -382,6 +452,21 @@ class NuitkaPluginBase(getMetaClassBase("Plugin", require_slots=False)):
         """Is this a detection plugin, i.e. one which is only there to inform."""
         return hasattr(cls, "detector_for")
 
+    @classmethod
+    def hasCategory(cls, category):
+        return category in cls.getCategories()
+
+    @classmethod
+    def getCategories(cls):
+        plugin_category = getattr(cls, "plugin_category", None)
+
+        if plugin_category is None:
+            result = ()
+        else:
+            result = plugin_category.split(",")
+
+        return OrderedSet(sorted(result))
+
     @classmethod
     def addPluginCommandLineOptions(cls, group):
         # Call group.add_option() here.
@@ -883,6 +968,7 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
         ignore_suffixes=(),
         only_suffixes=(),
         normalize=True,
+        raw=False,
     ):
         return makeIncludedDataDirectory(
             source_path=source_path,
@@ -895,6 +981,7 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
             ignore_suffixes=ignore_suffixes,
             only_suffixes=only_suffixes,
             normalize=normalize,
+            raw=raw,
         )
 
     def makeIncludedEmptyDirectory(self, dest_path, reason, tags):
@@ -1033,6 +1120,18 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
 
+    def decideAnnotations(self, module_name):
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
+    def decideDocStrings(self, module_name):
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
+    def decideAssertions(self, module_name):
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
     def getPreprocessorSymbols(self):
         """Decide which C defines to be used in compilation.
 
@@ -1193,21 +1292,25 @@ Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountere
             setup_codes = setup_codes.splitlines()
 
         if not setup_codes:
-            setup_codes = ["pass"]
+            setup_codes = ("pass",)
 
         cmd = r"""\
 from __future__ import print_function
 from __future__ import absolute_import
+import sys
 
 try:
 %(setup_codes)s
 except ImportError:
-    import sys
     sys.exit(38)
+try:
 %(query_codes)s
+except Exception as e:
+    sys.stderr.write("\n%%s" %% repr(e))
+    sys.exit(39)
 """ % {
             "setup_codes": "\n".join("   %s" % line for line in setup_codes),
-            "query_codes": "\n".join(query_codes),
+            "query_codes": "\n".join("   %s" % line for line in query_codes),
         }
 
         if shallShowExecutedCommands():
@@ -1223,7 +1326,17 @@ except ImportError:
                 return None
 
             if Options.is_debug:
-                self.info(cmd)
+                self.info(cmd, keep_format=True)
+
+            if e.returncode == 39:
+                # TODO: Recognize the ModuleNotFoundError or ImportError exceptions
+                # and output the missing module.
+                self.warning(
+                    "Exception during compile time command execution: %s"
+                    % e.stderr.splitlines()[-1]
+                )
+
+                return None
 
             raise
 
@@ -1364,16 +1477,22 @@ except ImportError:
             for count, variable_config in enumerate(
                 self.config.get(full_name, section="variables")
             ):
-                setup_codes = variable_config.get("setup_code")
-                declarations = variable_config.get("declarations")
+                environment = variable_config.get("environment", {})
+                setup_codes = variable_config.get("setup_code", [])
+                if type(setup_codes) is str:
+                    setup_codes = setup_codes.splitlines()
+                declarations = variable_config.get("declarations", {})
 
-                if declarations and self.evaluateCondition(
+                if len(declarations) < 1:
+                    self.sysexit(
+                        "Error, no variable 'declarations' for %s makes no sense."
+                        % full_name
+                    )
+
+                if self.evaluateCondition(
                     full_name=full_name,
                     condition=variable_config.get("when", "True"),
                 ):
-                    if type(setup_codes) is str:
-                        setup_codes = setup_codes.splitlines()
-
                     setup_codes.extend(
                         "%s=%r" % (constant_name, constant_value)
                         for (
@@ -1382,11 +1501,31 @@ except ImportError:
                         ) in self.getExpressionConstants(full_name=full_name).items()
                     )
 
-                    info = self.queryRuntimeInformationMultiple(
-                        "%s_variables_%s" % (full_name.asString(), count),
-                        setup_codes=setup_codes,
-                        values=tuple(declarations.items()),
-                    )
+                    env_variables = {}
+
+                    for env_name, env_value in environment.items():
+                        env_variables[env_name] = self.evaluateExpressionOrConstant(
+                            full_name=full_name,
+                            expression=env_value,
+                            config_name="variables config #%d" % count,
+                            extra_context=None,
+                            single_value=True,
+                        )
+
+                    with withEnvironmentVarsOverridden(env_variables):
+                        info = self.queryRuntimeInformationMultiple(
+                            "%s_variables_%s" % (full_name.asString(), count),
+                            setup_codes=setup_codes,
+                            values=tuple(declarations.items()),
+                        )
+
+                    if Options.isExperimental("display-yaml-variables"):
+                        self.info("Evaluated %r" % info)
+
+                    if info is None:
+                        self.sysexit(
+                            "Error, failed to evaluate variables for '%s'." % full_name
+                        )
 
                     variables.update(info.asDict())
 
@@ -1397,10 +1536,9 @@ except ImportError:
     def evaluateExpression(
         self, full_name, expression, config_name, extra_context, single_value
     ):
-        context = TagContext(logger=self, full_name=full_name, config_name=config_name)
-        context.update(control_tags)
-
-        context.update(_getEvaluationContext())
+        context = _makeEvaluationContext(
+            logger=self, full_name=full_name, config_name=config_name
+        )
 
         def get_variable(variable_name):
             assert type(variable_name) is str, variable_name
@@ -1458,38 +1596,46 @@ except ImportError:
                     raise
 
                 self.sysexit(
-                    "Error, failed to evaluate expression %r in this context, exception was '%s'."
+                    "Error, failed to evaluate expression %r in this context, exception was '%r'."
                     % (expression, e)
                 )
 
         if type(result) not in (str, unicode):
             if single_value:
-                self.sysexit(
-                    """\
-Error, expression '%s' for module '%s' did not evaluate to 'str' result."""
-                    % (expression, full_name)
+                self._checkStrResult(
+                    value=result, expression=expression, full_name=full_name
                 )
             else:
-                if type(result) not in (tuple, list):
-                    self.sysexit(
-                        """\
-Error, expression '%s' for module '%s' did not evaluate to 'str', 'tuple[str]' or 'list[str]' result."""
-                        % (expression, full_name)
-                    )
+                self._checkSequenceResult(
+                    value=result, expression=expression, full_name=full_name
+                )
 
                 for v in result:
-                    if type(v) not in (str, unicode):
-                        self.sysexit(
-                            """\
-Error, expression '%s' for module '%s' did not evaluate to 'str', 'tuple[str]' or 'list[str]' result."""
-                            % (expression, full_name)
-                        )
+                    self._checkStrResult(
+                        value=v, expression=expression, full_name=full_name
+                    )
 
                 # Make it immutable in case it's a list.
                 result = tuple(result)
 
         return result
 
+    def _checkStrResult(self, value, expression, full_name):
+        if type(value) not in (str, unicode):
+            self.sysexit(
+                """\
+Error, expression '%s' for module '%s' did not evaluate to 'str', 'tuple[str]' or 'list[str]' result, but '%s'"""
+                % (expression, full_name, type(value))
+            )
+
+    def _checkSequenceResult(self, value, expression, full_name):
+        if type(value) not in (tuple, list):
+            self.sysexit(
+                """\
+Error, expression '%s' for module '%s' did not evaluate to 'tuple[str]' or 'list[str]' result."""
+                % (expression, full_name)
+            )
+
     def evaluateCondition(self, full_name, condition):
         # Note: Caching makes no sense yet, this should all be very fast and
         # cache themselves. TODO: Allow plugins to contribute their own control
@@ -1499,12 +1645,10 @@ Error, expression '%s' for module '%s' did not evaluate to 'str', 'tuple[str]' o
         if condition == "False":
             return False
 
-        context = TagContext(
+        # TODO: Maybe add module name to config name?
+        context = _makeEvaluationContext(
             logger=self, full_name=full_name, config_name="'when' configuration"
         )
-        context.update(control_tags)
-
-        context.update(_getEvaluationContext())
 
         def get_parameter(parameter_name, default):
             result = Options.getModuleParameter(full_name, parameter_name)
@@ -1587,6 +1731,11 @@ Error, expression '%s' for module '%s' did not evaluate to 'str', 'tuple[str]' o
     def info(cls, message, keep_format=False):
         plugins_logger.info(message, prefix=cls.plugin_name, keep_format=keep_format)
 
+    @classmethod
+    def debug(cls, message, keep_format=False):
+        if Options.is_debug:
+            cls.info(message, keep_format=keep_format)
+
     @classmethod
     def sysexit(cls, message, mnemonic=None, reporting=True):
         plugins_logger.sysexit(
@@ -1594,94 +1743,6 @@ Error, expression '%s' for module '%s' did not evaluate to 'str', 'tuple[str]' o
         )
 
 
-class NuitkaYamlPluginBase(NuitkaPluginBase):
-    """Nuitka base class for all plugins that use yaml config"""
-
-    def __init__(self):
-        self.config = getYamlPackageConfiguration()
-
-    def getYamlConfigItem(
-        self, module_name, section, item_name, decide_relevant, default, recursive
-    ):
-        while True:
-            module_configs = self.config.get(module_name, section=section)
-
-            if module_configs is not None:
-                for module_config in module_configs:
-                    config_item = module_config.get(item_name, default)
-
-                    # Avoid condition, if the item is not relevant
-                    if decide_relevant is not None and not decide_relevant(config_item):
-                        continue
-
-                    if not self.evaluateCondition(
-                        full_name=module_name,
-                        condition=module_config.get("when", "True"),
-                    ):
-                        continue
-
-                    if recursive:
-                        yield module_name, config_item
-                    else:
-                        yield config_item
-
-            if not recursive:
-                break
-
-            module_name = module_name.getPackageName()
-            if not module_name:
-                break
-
-    def getYamlConfigItemItems(
-        self, module_name, section, item_name, decide_relevant, recursive
-    ):
-        def dict_decide_relevant(item_dict):
-            if not item_dict:
-                return False
-
-            if decide_relevant is None:
-                return True
-
-            for key, value in item_dict.items():
-                if decide_relevant(key, value):
-                    return True
-
-            return False
-
-        for item_config in self.getYamlConfigItem(
-            module_name=module_name,
-            section=section,
-            item_name=item_name,
-            decide_relevant=dict_decide_relevant,
-            default={},
-            recursive=recursive,
-        ):
-            if recursive:
-                for key, value in item_config[1].items():
-                    if decide_relevant(key, value):
-                        yield item_config[0], key, value
-            else:
-                for key, value in item_config.items():
-                    if decide_relevant(key, value):
-                        yield key, value
-
-
-def standalone_only(func):
-    """For plugins that have functionality that should be done in standalone mode only."""
-
-    @functools.wraps(func)
-    def wrapped(*args, **kwargs):
-        if isStandaloneMode():
-            return func(*args, **kwargs)
-        else:
-            if inspect.isgeneratorfunction(func):
-                return ()
-            else:
-                return None
-
-    return wrapped
-
-
 class TagContext(dict):
     def __init__(self, logger, full_name, config_name):
         dict.__init__(self)
@@ -1702,12 +1763,44 @@ class TagContext(dict):
             if key.startswith("use_"):
                 return False
 
+            if key == "no_asserts":
+                # TODO: This should be better decoupled.
+                from .Plugins import Plugins
+
+                return Plugins.decideAssertions(self.full_name) is False
+
+            if key == "no_docstrings":
+                from .Plugins import Plugins
+
+                return Plugins.decideDocStrings(self.full_name) is False
+
+            if key == "no_annotations":
+                from .Plugins import Plugins
+
+                return Plugins.decideAnnotations(self.full_name) is False
+
             self.logger.sysexit(
                 "Identifier '%s' in %s of module '%s' is unknown."
                 % (key, self.config_name, self.full_name)
             )
 
 
+def standalone_only(func):
+    """For plugins that have functionality that should be done in standalone mode only."""
+
+    @functools.wraps(func)
+    def wrapped(*args, **kwargs):
+        if isStandaloneMode():
+            return func(*args, **kwargs)
+        else:
+            if inspect.isgeneratorfunction(func):
+                return ()
+            else:
+                return None
+
+    return wrapped
+
+
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
